package org.xbib.jdbc.query;

import java.util.function.Supplier;

/**
 * A block of runnable code using a transacted Database.
 */
@FunctionalInterface
public interface DbCodeTx {
    /**
     * Implement this method to provide a block of code that uses the provided database
     * and is transacted. Whether the transaction will commit or rollback is typically
     * controlled by the code that invokes this method.
     *
     * <p>If a {@link Throwable} is thrown from this method, it will be caught, wrapped in
     * a DatabaseException (if it is not already one), and then propagated.</p>
     */
    void run(Supplier<Database> db, Transaction tx) throws Exception;
}
