package org.xbib.jdbc.query;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

/**
 * Control various optional behavior for the database interactions.
 */
public class OptionsDefault implements Options {

    private final Flavor flavor;

    public OptionsDefault(Flavor flavor) {
        this.flavor = flavor;
    }

    @Override
    public boolean allowTransactionControl() {
        return false;
    }

    @Override
    public boolean ignoreTransactionControl() {
        return false;
    }

    @Override
    public boolean allowConnectionAccess() {
        return false;
    }

    @Override
    public boolean isLogParameters() {
        return false;
    }

    @Override
    public boolean isDetailedExceptions() {
        return false;
    }

    @Override
    public String generateErrorCode() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd:H:m:s");
        return sdf.format(new Date()) + "-" + Math.round(Math.random() * 1000000);
    }

    @Override
    public boolean useBytesForBlob() {
        return flavor().useBytesForBlob();
    }

    @Override
    public boolean useStringForClob() {
        return flavor().useStringForClob();
    }

    @Override
    public Flavor flavor() {
        return flavor;
    }

    @Override
    public LocalDateTime currentDate() {
        return LocalDateTime.now();
    }

    @Override
    public boolean useDatePerAppOnly() {
        return false;
    }

    @Override
    public Calendar calendarForTimestamps() {
        return Calendar.getInstance(TimeZone.getDefault());
    }

    @Override
    public int maxStringLengthParam() {
        return 4000;
    }
}
