package org.xbib.jdbc.query;

import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.Date;

/**
 * Base class for selectively overriding another Options object.
 */
public class OptionsOverride implements Options {

    private Options parent;

    /**
     * Wrap another {@code Options} and defer to it for anything we choose not
     * to override.
     */
    public OptionsOverride(Options parent) {
        this.parent = parent;
    }

    /**
     * Defer to OptionsDefault for anything that is not specified, and use postgresql flavor.
     */
    public OptionsOverride() {
        parent = new OptionsDefault(Flavor.postgresql);
    }

    /**
     * Defer to OptionsDefault for anything that is not specified, using the specified flavor.
     */
    public OptionsOverride(Flavor flavor) {
        parent = new OptionsDefault(flavor);
    }

    public void setParent(Options parent) {
        this.parent = parent;
    }

    public OptionsOverride withParent(Options parent) {
        this.parent = parent;
        return this;
    }

    @Override
    public boolean allowTransactionControl() {
        return parent.allowTransactionControl();
    }

    @Override
    public boolean ignoreTransactionControl() {
        return parent.ignoreTransactionControl();
    }

    @Override
    public boolean allowConnectionAccess() {
        return parent.allowConnectionAccess();
    }

    @Override
    public boolean isLogParameters() {
        return parent.isLogParameters();
    }

    @Override
    public boolean isDetailedExceptions() {
        return parent.isDetailedExceptions();
    }

    @Override
    public String generateErrorCode() {
        return parent.generateErrorCode();
    }

    @Override
    public boolean useBytesForBlob() {
        return parent.useBytesForBlob();
    }

    @Override
    public boolean useStringForClob() {
        return parent.useStringForClob();
    }

    @Override
    public Flavor flavor() {
        return parent.flavor();
    }

    @Override
    public LocalDateTime currentDate() {
        return parent.currentDate();
    }

    @Override
    public boolean useDatePerAppOnly() {
        return parent.useDatePerAppOnly();
    }

    @Override
    public Calendar calendarForTimestamps() {
        return parent.calendarForTimestamps();
    }

    @Override
    public int maxStringLengthParam() {
        return parent.maxStringLengthParam();
    }
}
