package org.xbib.jdbc.query;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * Interface for configuring (setting parameters) and executing a chunk of SQL.
 */
public interface SqlSelect {
    
    
    SqlSelect argBoolean(Boolean arg);

    
    
    SqlSelect argBoolean( String argName, Boolean arg);

    
    
    SqlSelect argInteger(Integer arg);

    
    
    SqlSelect argInteger( String argName, Integer arg);

    
    
    SqlSelect argLong(Long arg);

    
    
    SqlSelect argLong( String argName, Long arg);

    
    
    SqlSelect argFloat(Float arg);

    
    
    SqlSelect argFloat( String argName, Float arg);

    
    
    SqlSelect argDouble(Double arg);

    
    
    SqlSelect argDouble( String argName, Double arg);

    
    
    SqlSelect argBigDecimal(BigDecimal arg);

    
    
    SqlSelect argBigDecimal( String argName, BigDecimal arg);

    
    
    SqlSelect argString(String arg);

    
    
    SqlSelect argString( String argName, String arg);

    
    
    SqlSelect argDate(LocalDateTime arg);  // Date with time

    SqlSelect argDate(String argName, LocalDateTime arg);  // Date with time

    
    
    SqlSelect argLocalDate(LocalDate arg); // Date without time

    
    
    SqlSelect argLocalDate( String argName, LocalDate arg); // Date without time

    
    
    SqlSelect argDateNowPerApp();

    
    
    SqlSelect argDateNowPerApp( String argName);

    
    
    SqlSelect argDateNowPerDb();

    
    
    SqlSelect argDateNowPerDb( String argName);

    
    
    SqlSelect withTimeoutSeconds(int seconds);

    
    
    SqlSelect withMaxRows(int rows);

    
    
    SqlSelect withArgs(SqlArgs args);

    
    
    SqlSelect apply(Apply apply);

    
    
    SqlSelect fetchSize(int fetchSize);

    
    
    Boolean queryBooleanOrNull();

    
    boolean queryBooleanOrFalse();

    
    boolean queryBooleanOrTrue();

    
    
    Long queryLongOrNull();

    
    long queryLongOrZero();

    /**
     * Shorthand for reading numbers from the first column of the result.
     *
     * @return the first column values, omitting any that were null
     */
    
    
    List<Long> queryLongs();

    
    
    Integer queryIntegerOrNull();

    
    int queryIntegerOrZero();

    
    
    List<Integer> queryIntegers();

    
    
    Float queryFloatOrNull();

    
    float queryFloatOrZero();

    
    
    List<Float> queryFloats();

    
    
    Double queryDoubleOrNull();

    
    double queryDoubleOrZero();

    
    
    List<Double> queryDoubles();

    
    
    BigDecimal queryBigDecimalOrNull();

    
    
    BigDecimal queryBigDecimalOrZero();

    
    
    List<BigDecimal> queryBigDecimals();

    
    
    String queryStringOrNull();

    
    
    String queryStringOrEmpty();

    /**
     * Shorthand for reading strings from the first column of the result.
     *
     * @return the first column values, omitting any that were null
     */
    
    
    List<String> queryStrings();

    
    
    LocalDateTime queryDateOrNull();  // Date with time

    
    
    List<LocalDateTime> queryDates();  // Date with time

    
    
    LocalDate queryLocalDateOrNull();  // Date without time

    
    
    List<LocalDate> queryLocalDates();  // Date without time

    /**
     * This is the most generic and low-level way to iterate the query results.
     * Consider using one of the other methods that can handle the iteration for you.
     *
     * @param rowsHandler the process() method of this handler will be called once
     *                    and it will be responsible for iterating the results
     */
    <T> T query(RowsHandler<T> rowsHandler);

    /**
     * Query zero or one row. If zero rows are available a null will be returned.
     * If more than one row is available a {@link ConstraintViolationException}
     * will be thrown.
     *
     * @param rowHandler the process() method of this handler will be called once
     *                   if there are results, or will not be called if there are
     *                   no results
     */
    <T> T queryOneOrNull(RowHandler<T> rowHandler);

    /**
     * Query exactly one row. If zero rows are available or more than one row is
     * available a {@link ConstraintViolationException} will be thrown.
     *
     * @param rowHandler the process() method of this handler will be called once
     *                   if there are results, or will not be called if there are
     *                   no results
     */
    <T> T queryOneOrThrow(RowHandler<T> rowHandler);

    /**
     * Query zero or one row. If zero rows are available a null will be returned.
     * If more than one row is available the first row will be returned.
     *
     * @param rowHandler the process() method of this handler will be called once
     *                   if there are results (for the first row), or will not be
     *                   called if there are no results
     */
    <T> T queryFirstOrNull(RowHandler<T> rowHandler);

    /**
     * Query zero or one row. If zero rows are available a {@link ConstraintViolationException}
     * will be thrown. If more than one row is available the first row will be returned.
     *
     * @param rowHandler the process() method of this handler will be called once
     *                   if there are results (for the first row), or will not be
     *                   called if there are no results
     */
    <T> T queryFirstOrThrow(RowHandler<T> rowHandler);

    /**
     * Query zero or more rows. If zero rows are available an empty list will be returned.
     * If one or more rows are available each row will be read and added to a list, which
     * is returned.
     *
     * @param rowHandler the process() method of this handler will be called once
     *                   for each row in the result, or will not be called if there are
     *                   no results. Only non-null values returned will be added to the
     *                   result list.
     */
    <T> List<T> queryMany(RowHandler<T> rowHandler);

    interface Apply {
        void apply(SqlSelect select);
    }
}
