package org.xbib.jdbc.query;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * Interface for configuring (setting parameters) and executing a chunk of SQL.
 */
public interface SqlUpdate {
    
    SqlUpdate argBoolean(Boolean arg);
    
    SqlUpdate argBoolean( String argName, Boolean arg);
    
    SqlUpdate argInteger(Integer arg);
    
    SqlUpdate argInteger( String argName, Integer arg);
    
    SqlUpdate argLong(Long arg);
    
    SqlUpdate argLong( String argName, Long arg);
    
    SqlUpdate argFloat(Float arg);
    
    SqlUpdate argFloat( String argName, Float arg);
    
    SqlUpdate argDouble(Double arg);
    
    SqlUpdate argDouble( String argName, Double arg);
    
    SqlUpdate argBigDecimal(BigDecimal arg);

    SqlUpdate argBigDecimal( String argName, BigDecimal arg);
    
    SqlUpdate argString(String arg);
    
    SqlUpdate argString( String argName, String arg);
    
    SqlUpdate argDate(LocalDateTime arg);
    
    SqlUpdate argDate( String argName, LocalDateTime arg);

    SqlUpdate argLocalDate(LocalDate arg);

    SqlUpdate argLocalDate( String argName, LocalDate arg);

    SqlUpdate argDateNowPerApp();
    
    SqlUpdate argDateNowPerApp( String argName);

    SqlUpdate argDateNowPerDb();

    SqlUpdate argDateNowPerDb( String argName);

    SqlUpdate argBlobBytes(byte[] arg);

    SqlUpdate argBlobBytes( String argName, byte[] arg);

    SqlUpdate argBlobStream(InputStream arg);

    SqlUpdate argBlobStream( String argName, InputStream arg);

    SqlUpdate argClobString(String arg);

    SqlUpdate argClobString( String argName, String arg);

    SqlUpdate argClobReader(Reader arg);

    SqlUpdate argClobReader( String argName, Reader arg);
    
    SqlUpdate withArgs(SqlArgs args);
    
    SqlUpdate apply(Apply apply);

    /**
     * Execute the SQL update and return the number of rows was affected.
     */
    int update();

    /**
     * Execute the SQL update and check that the expected number of rows was affected.
     *
     * @throws WrongNumberOfRowsException if the number of rows affected did not match
     *                                    the value provided
     */
    void update(int expectedRowsUpdated);

    interface Apply {
        void apply(SqlUpdate update);
    }
}
