package org.xbib.jdbc.query;

import java.util.Objects;

/**
 * Convenience for conditional SQL generation.
 */
public class When {

    private final Flavor actualFlavor;

    private String chosen;

    public When(Flavor actualFlavor) {
        this.actualFlavor = actualFlavor;
    }
    
    public When oracle(String sql) {
        if (actualFlavor == Flavor.oracle) {
            chosen = sql;
        }
        return this;
    }
    
    public When derby(String sql) {
        if (actualFlavor == Flavor.derby) {
            chosen = sql;
        }
        return this;
    }
    
    public When postgres(String sql) {
        if (actualFlavor == Flavor.postgresql) {
            chosen = sql;
        }
        return this;
    }
    
    public When sqlserver(String sql) {
        if (actualFlavor == Flavor.sqlserver) {
            chosen = sql;
        }
        return this;
    }

    
    public String other(String sql) {
        if (chosen == null) {
            chosen = sql;
        }
        return chosen;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        When when = (When) o;
        return Objects.equals(chosen, when.chosen) && actualFlavor == when.actualFlavor;
    }

    @Override
    public int hashCode() {
        return Objects.hash(chosen, actualFlavor);
    }

    @Override
    public String toString() {
        return chosen == null ? "" : chosen;
    }
}
