package org.xbib.jdbc.query.test;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.xbib.jdbc.query.DatabaseProvider;
import org.xbib.jdbc.query.OptionsOverride;
import org.xbib.jdbc.query.Schema;

import java.math.BigDecimal;
import java.util.logging.Logger;

import static org.junit.jupiter.api.Assertions.assertEquals;


/**
 * Exercise Database functionality with a real database (Derby).
 */
public class DerbyTest extends CommonTest {

    private static final Logger logger = Logger.getLogger(DerbyTest.class.getName());

    static {
        System.setProperty("derby.stream.error.file", System.getProperty("user.dir") + "/build/derby-errors.log");
    }

    @Override
    protected DatabaseProvider createDatabaseProvider(OptionsOverride options) {
        return DatabaseProvider.fromDriverManager("jdbc:derby:build/testdb;create=true")
                .withSqlParameterLogging().withSqlInExceptionMessages().withOptions(options).create();
    }

    // TODO fix this test
    @Disabled("Not sure why this fails on the build servers right now...")
    @Test
    public void clockSync() {
        super.clockSync();
    }

    @Disabled("Derby prohibits NaN and Infinity (https://issues.apache.org/jira/browse/DERBY-3290)")
    @Test
    public void argFloatNaN() {
        super.argFloatNaN();
    }

    @Disabled("Derby prohibits NaN and Infinity (https://issues.apache.org/jira/browse/DERBY-3290)")
    @Test
    public void argFloatInfinity() {
        super.argFloatInfinity();
    }

    @Disabled("Derby prohibits NaN and Infinity (https://issues.apache.org/jira/browse/DERBY-3290)")
    @Test
    public void argDoubleNaN() {
        super.argDoubleNaN();
    }

    @Disabled("Derby prohibits NaN and Infinity (https://issues.apache.org/jira/browse/DERBY-3290)")
    @Test
    public void argDoubleInfinity() {
        super.argDoubleInfinity();
    }

    @Disabled("Current Derby behavior is to convert -0f to 0f")
    @Test
    public void argFloatNegativeZero() {
        super.argFloatNegativeZero();
    }

    @Disabled("Current Derby behavior is to convert -0d to 0d")
    @Test
    public void argDoubleNegativeZero() {
        super.argDoubleNegativeZero();
    }

    @Disabled("Derby does not support timestamp intervals")
    @Test
    public void intervals() {
        super.intervals();
    }

    @Test
    public void argBigDecimal31Precision0() {
        db.dropTableQuietly("dbtest");
        new Schema().addTable("dbtest").addColumn("i").asBigDecimal(31, 0).schema().execute(db);
        BigDecimal value = new BigDecimal("9999999999999999999999999999999"); // 31 digits
        db.toInsert("insert into dbtest (i) values (?)").argBigDecimal(value).insert(1);
        assertEquals(value,
                db.toSelect("select i from dbtest where i=?").argBigDecimal(value).queryBigDecimalOrNull());
    }

    @Test
    public void argBigDecimal31Precision1() {
        db.dropTableQuietly("dbtest");
        new Schema().addTable("dbtest").addColumn("i").asBigDecimal(31, 1).schema().execute(db);
        BigDecimal value = new BigDecimal("999999999999999999999999999999.9"); // 31 digits
        db.toInsert("insert into dbtest (i) values (?)").argBigDecimal(value).insert(1);
        assertEquals(value,
                db.toSelect("select i from dbtest where i=?").argBigDecimal(value).queryBigDecimalOrNull());
    }

    @Test
    public void argBigDecimal31Precision30() {
        db.dropTableQuietly("dbtest");
        new Schema().addTable("dbtest").addColumn("i").asBigDecimal(31, 30).schema().execute(db);
        BigDecimal value = new BigDecimal("9.999999999999999999999999999999"); // 31 digits
        db.toInsert("insert into dbtest (i) values (?)").argBigDecimal(value).insert(1);
        assertEquals(value,
                db.toSelect("select i from dbtest where i=?").argBigDecimal(value).queryBigDecimalOrNull());
    }

    @Test
    public void argBigDecimal31Precision31() {
        db.dropTableQuietly("dbtest");
        new Schema().addTable("dbtest").addColumn("i").asBigDecimal(31, 31).schema().execute(db);
        BigDecimal value = new BigDecimal("0.9999999999999999999999999999999"); // 31 digits
        db.toInsert("insert into dbtest (i) values (?)").argBigDecimal(value).insert(1);
        System.out.println(db.toSelect("select i from dbtest").queryBigDecimalOrNull());
        assertEquals(value,
                db.toSelect("select i from dbtest where i=?").argBigDecimal(value).queryBigDecimalOrNull());
    }

    @Disabled("Derby limits out at precision 31")
    @Test
    public void argBigDecimal38Precision0() {
        super.argBigDecimal38Precision0();
    }

    @Disabled("Derby limits out at precision 31")
    @Test
    public void argBigDecimal38Precision1() {
        super.argBigDecimal38Precision1();
    }

    @Disabled("Derby limits out at precision 31")
    @Test
    public void argBigDecimal38Precision37() {
        super.argBigDecimal38Precision37();
    }

    @Disabled("Derby limits out at precision 31")
    @Test
    public void argBigDecimal38Precision38() {
        super.argBigDecimal38Precision38();
    }
}
